#' Summary method for `bayes_mode` objects
#' 
#' @param object An object of class `bayes_mode`.
#' @param ... Not used.
#' 
#' @export
summary.bayes_mode <- function(object, ...) {
  modes = object$modes
  
  p1 = object$p1
  cat("The posterior probability of the data being multimodal is", 1-p1)
  
  tb_nb_modes = t(object$tb_nb_modes)
  colnames(tb_nb_modes) = c("Number of modes", "Posterior probabilty")
  tb_nb_modes = tb_nb_modes[order(tb_nb_modes[, 1]), ]

  cat("\n\n Number of estimated modes and their posterior probabilities:\n")
  tb_nb_modes
}


#' Summary method for `mix_mode` objects
#' 
#' @param object An object of class `mix_mode`.
#' @param ... Not used.
#' 
#' @export
summary.mix_mode <- function(object, ...) {
  Nb_m = length(object$mode_estimates)
  algo = object$algo
  d = object$dist
  K = object$K
  
  if (is.na(d)) {
    d = object$dist_type
  }
  
  if (Nb_m == 1) {
    m = "Mode"
  } else {
    m = "Modes"
  }
  
  cat("\n",m, "of a", d, "mixture with", K, "components.")
  cat("\n- Number of modes found:", Nb_m)
  cat("\n- Mode estimation technique:", object$algo, "algorithm")
}


#' Summary method for `mixture` objects
#' 
#' @param object An object of class `mixture`.
#' @param ... Not used.
#' 
#' @export
summary.mixture <- function(object, ...) {
  cat("\n Estimated mixture distribution.")
  cat("\n- Mixture type:", object$dist_type)
  cat("\n- Number of components:", object$K)
  cat("\n- Distribution family:", object$dist)
  cat("\n- Number of distribution variables:", object$nb_var)
  cat("\n- Names of variables:",
      object$pars_names[object$pars_names!="eta"])
}



#' Summary method for `bayes_mixture` objects
#' The summary of MCMC draws is given by the function
#' `summarise_draws` from package \pkg{posterior}.
#' @param object An object of class `bayes_mixture`.
#' @param ... Not used.
#' 
#' @importFrom posterior summarise_draws
#' 
#' @export
summary.bayes_mixture <- function(object, ...) {
  d = object$dist
  K = object$K
  
  if (is.na(d)) {
    d = object$dist_type
  }
  
  cat("\n Mixture estimated with a Bayesian MCMC method.")
  cat("\n- Mixture type:", object$dist_type)
  cat("\n- Number of components:", object$K)
  cat("\n- Distribution family:", object$dist)
  cat("\n- Number of distribution variables:", object$nb_var)
  cat("\n- Names of variables:",
      object$pars_names[object$pars_names!="eta"])
  
  cat("\n\nSummary of MCMC output after burnin:\n")
  print(summarise_draws(object$mcmc))
  cat(paste0("this table can be reproduced with: summarise_draws(",deparse(substitute(object)),"$mcmc)"))
}
